// CFileToWAV.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "CFileToWAV.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

// The one and only application object

CWinApp theApp;

using namespace std;

typedef struct auxi
{
	SYSTEMTIME timeStart;
	SYSTEMTIME timeEnd;
	long lFreq1;
	BYTE padding[24];
	long lFreq2;
} AUXI, *PAUXI;

class ScheduleEntry
{
public:
	double fDuration;
	double fFreq;
	CString strName;
	double fBandwidth;
};

typedef map<string,ScheduleEntry> ScheduleEntryMap;

int _tmain(int argc, TCHAR* argv[], TCHAR* envp[])
{
	int nRetCode = 0;

	// initialize MFC and print and error on failure
	if (!AfxWinInit(::GetModuleHandle(NULL), NULL, ::GetCommandLine(), 0))
	{
		_tprintf(_T("Fatal Error: MFC initialization failed\n"));
		return 1;
	}

	CStdioFile fileSchedule;

	if (fileSchedule.Open(_T("schedule.txt"), CFile::modeRead | CFile::typeText) == FALSE)
	{
		cout << "Failed to open schedule.txt" << endl;
		return 1;
	}

	ScheduleEntryMap mapSchedule;

	CString str;
	while (fileSchedule.ReadString(str))
	{
		str.Trim();

		if (str.IsEmpty())
			continue;

		if (str[0] == _T('#'))
			continue;

		CStringArray array;
		if (Teh::Utils::Tokenise(str, array, _T(' '), NULL, true) < 3)
			continue;

		ScheduleEntry e;
		e.fDuration = _tstof(array[0]);
		e.fFreq = _tstof(array[1]);
		e.strName = array[2];
		if (array.GetCount() > 3)
			e.fBandwidth = _tstof(array[3]);
		else
			e.fBandwidth = 1.0;

		mapSchedule[string(CStringA(e.strName))] = e;

		cout << CStringA(e.strName) << ": " << e.fFreq << " MHz @ " << e.fBandwidth << " Msps" << endl;
	}

	fileSchedule.Close();

	cout << endl;

	///////////////////////////////////////////////////////////////////////////

	//cout << "Enter destination path: ";
	string strDestination;
	//getline(cin, strDestination);

	const UINT nBufferSize = 4096;
	LPBYTE pBuffer = new BYTE[nBufferSize];

	for (int i = 1; i < argc; ++i)
	{
		//cout << string(argv[i]) << endl;
		_tprintf(_T("%s\n"), argv[i]);

		CString strPath(argv[i]);

		CFile fileIn;
		if (fileIn.Open(strPath, CFile::modeRead) == FALSE)
		{
			cout << "Failed to open" << endl;
			continue;
		}

		UINT nDataLength = (UINT)fileIn.GetLength();
		cout << nDataLength << " bytes"/* << endl*/;

		if (nDataLength == 0)
		{
			cout << endl;
			continue;
		}

		CFileStatus status;
		fileIn.GetStatus(status);

		///////////////////////////////////////////////////////////////////////////

		CString strKey(Teh::Utils::GetFileTitle(strPath));
		int iIndex = strKey.ReverseFind(_T('-'));
		if (iIndex > -1)
			strKey = strKey.Left(iIndex);

		double fFrequency = 0, fSampleRate = 0;

		ScheduleEntryMap::iterator it = mapSchedule.find(string(CStringA(strKey)));
		if (it != mapSchedule.end())
		{
			fSampleRate = it->second.fBandwidth * 1e6;
			fFrequency = it->second.fFreq * 1e6;
		}

		if (fSampleRate == 0)
		{
			string strSamplesPerSec;
			cout << "Sample rate (MHz): ";
			cin >> strSamplesPerSec;
			fSampleRate = atof(strSamplesPerSec.c_str()) * 1e6;
		}

		if (fFrequency == 0)
		{
			string strFrequency;
			cout << "Frequency (MHz): ";
			cin >> strFrequency;
			fFrequency = atof(strFrequency.c_str()) * 1e6;
		}

		UINT nFrequency = (UINT)fFrequency, nSampleRate = (UINT)fSampleRate;

		if (nSampleRate == 0)
			continue;

		if (nFrequency == 0)
			continue;

		cout << " " << "(" << (fFrequency / 1e6f) << " MHz @ " << (fSampleRate / 1e6f) << " Msps)";
		
		CString strDestinationPath(strDestination.c_str());
		if (strDestinationPath.IsEmpty() == false)
			Teh::Utils::AddTrailingSlash(strDestinationPath);
		strPath = strDestinationPath + Teh::Utils::GetFileName(strPath) + _T(".wav");
		cout << " " << "-> " << (CStringA)strPath << endl;

		CFile fileOut;
		if (fileOut.Open(strPath, CFile::modeCreate | CFile::modeWrite) == FALSE)
		{
			cout << "Failed to create" << endl;
			continue;
		}

		FOURCC dw = MAKEFOURCC('R','I','F','F');
		fileOut.Write(&dw, sizeof(dw));

		UINT nTotalSize =	sizeof(FOURCC) +	// WAVE
			sizeof(FOURCC) + sizeof(UINT) + sizeof(WAVEFORMATEX) +	// fmt
			sizeof(FOURCC) + sizeof(UINT) + 164 +					// auxi
			sizeof(FOURCC) + sizeof(UINT) + nDataLength;			// data

		fileOut.Write(&nTotalSize, sizeof(UINT));

		dw = MAKEFOURCC('W','A','V','E');
		fileOut.Write(&dw, sizeof(dw));

		dw = MAKEFOURCC('f','m','t',' ');
		fileOut.Write(&dw, sizeof(dw));

		dw = sizeof(WAVEFORMATEX);
		fileOut.Write(&dw, sizeof(dw));

		WAVEFORMATEX wfx;
		//PCMWAVEFORMAT wfx;
		ZeroMemory(&wfx, sizeof(wfx));
		wfx.cbSize = 0;
		wfx.wFormatTag = WAVE_FORMAT_PCM;
		wfx.nChannels = 2;
		wfx.nSamplesPerSec = nSampleRate;
		wfx.wBitsPerSample = 16;
		wfx.nBlockAlign = (wfx.wBitsPerSample / 2) * wfx.nChannels;
		wfx.nAvgBytesPerSec = nSampleRate * wfx.nBlockAlign;

		fileOut.Write(&wfx, sizeof(wfx));

		dw = MAKEFOURCC('a','u','x','i');
		fileOut.Write(&dw, sizeof(dw));

		const UINT auxiSize = 164;

		fileOut.Write(&auxiSize, sizeof(auxiSize));

		PAUXI pAuxi = (PAUXI)new BYTE[auxiSize];
		ZeroMemory(pAuxi, auxiSize);

		pAuxi->lFreq1 = pAuxi->lFreq2 = nFrequency;

		//status.m_ctime.GetAsSystemTime(pAuxi->timeStart);

		FILETIME ftCreate;
		GetFileTime(fileIn.m_hFile, &ftCreate, NULL, NULL);
		FileTimeToSystemTime(&ftCreate, &pAuxi->timeStart);

		CTime time(status.m_ctime);
		double d = (double)nDataLength / (double)nSampleRate;
		time += CTimeSpan((int)floor(d));
		time.GetAsSystemTime(pAuxi->timeEnd);

		d -= floor(d);
		ASSERT(pAuxi->timeEnd.wMilliseconds == 0);
		pAuxi->timeEnd.wMilliseconds += pAuxi->timeStart.wMilliseconds + (WORD)(d * 1000.0);
		if (pAuxi->timeEnd.wMilliseconds >= 1000)
		{
			time += CTimeSpan(pAuxi->timeEnd.wMilliseconds / 1000);
			WORD ms = pAuxi->timeEnd.wMilliseconds % 1000;
			time.GetAsSystemTime(pAuxi->timeEnd);

			pAuxi->timeEnd.wMilliseconds = ms;
		}

		fileOut.Write(pAuxi, auxiSize);

		delete [] pAuxi;

		dw = MAKEFOURCC('d','a','t','a');
		fileOut.Write(&dw, sizeof(dw));

		fileOut.Write(&nDataLength, sizeof(nDataLength));

		UINT nCount = 0;
		UINT nLimit = (100 * 1024 * 1024) / nBufferSize;
		UINT nRead;

		while (nRead = fileIn.Read(pBuffer, nBufferSize))
		{
			nCount++;
			if (nCount == nRead)
			{
				printf(".");
				fflush(stdout);
				nCount = 0;
			}

			fileOut.Write(pBuffer, nRead);
		}

		fileOut.Close();
		fileIn.Close();

		cout << endl;
	}

	delete [] pBuffer;

	return nRetCode;
}
